@@ -19,7 +19,6 @@ package cl_test
 import (
 	"bytes"
 	"os"
-	"path"
 	"testing"
 
 	"github.com/goplus/gop/cl"
@@ -42,11 +41,22 @@ func lookupClass(ext string) (c *modfile.Project, ok bool) {
 			Works: []*modfile.Class{{Ext: ".t2spx", Class: "Sprite"},
 				{Ext: ".t2spx2", Class: "Sprite2"}},
 			PkgPaths: []string{"github.com/goplus/gop/cl/internal/spx2"}}, true
-	case ".t3spx", ".t3spx2":
+	case "_t3spx.gox", ".t3spx2":
 		return &modfile.Project{
-			Works: []*modfile.Class{{Ext: ".t3spx", Class: "Sprite"},
+			Works: []*modfile.Class{{Ext: "_t3spx.gox", Class: "Sprite"},
 				{Ext: ".t3spx2", Class: "Sprite2"}},
 			PkgPaths: []string{"github.com/goplus/gop/cl/internal/spx2"}}, true
+	case "_spx.gox":
+		return &modfile.Project{
+			Ext: "_spx.gox", Class: "Game",
+			Works:    []*modfile.Class{{Ext: "_spx.gox", Class: "Sprite"}},
+			PkgPaths: []string{"github.com/goplus/gop/cl/internal/spx3", "math"},
+			Import:   []*modfile.Import{{Path: "github.com/goplus/gop/cl/internal/spx3/jwt"}}}, true
+	case "_xtest.gox":
+		return &modfile.Project{
+			Ext: "_xtest.gox", Class: "App",
+			Works:    []*modfile.Class{{Ext: "_xtest.gox", Class: "Case"}},
+			PkgPaths: []string{"github.com/goplus/gop/test", "testing"}}, true
 	}
 	return
 }
@@ -54,10 +64,10 @@ func lookupClass(ext string) (c *modfile.Project, ok bool) {
 func spxParserConf() parser.Config {
 	return parser.Config{
 		ClassKind: func(fname string) (isProj bool, ok bool) {
-			ext := path.Ext(fname)
+			ext := modfile.ClassExt(fname)
 			c, ok := lookupClass(ext)
 			if ok {
-				isProj = (c.Ext == ext)
+				isProj = c.IsProj(ext, fname)
 			}
 			return
 		},
@@ -69,10 +79,14 @@ func gopSpxTest(t *testing.T, gmx, spxcode, expected string) {
 }
 
 func gopSpxTestEx(t *testing.T, gmx, spxcode, expected, gmxfile, spxfile string) {
-	gopSpxTestExConf(t, "gopSpxTest", gblConf, gmx, spxcode, expected, gmxfile, spxfile)
+	gopSpxTestExConf(t, "gopSpxTest", gblConf, gmx, spxcode, expected, gmxfile, spxfile, "")
 }
 
-func gopSpxTestExConf(t *testing.T, name string, conf *cl.Config, gmx, spxcode, expected, gmxfile, spxfile string) {
+func gopSpxTestEx2(t *testing.T, gmx, spxcode, expected, gmxfile, spxfile, resultFile string) {
+	gopSpxTestExConf(t, "gopSpxTest", gblConf, gmx, spxcode, expected, gmxfile, spxfile, resultFile)
+}
+
+func gopSpxTestExConf(t *testing.T, name string, conf *cl.Config, gmx, spxcode, expected, gmxfile, spxfile, resultFile string) {
 	t.Run(name, func(t *testing.T) {
 		cl.SetDisableRecover(true)
 		defer cl.SetDisableRecover(false)
@@ -89,7 +103,7 @@ func gopSpxTestExConf(t *testing.T, name string, conf *cl.Config, gmx, spxcode,
 			t.Fatal("NewPackage:", err)
 		}
 		var b bytes.Buffer
-		err = pkg.WriteTo(&b)
+		err = pkg.WriteTo(&b, resultFile)
 		if err != nil {
 			t.Fatal("gox.WriteTo failed:", err)
 		}
@@ -108,10 +122,9 @@ func gopSpxErrorTestEx(t *testing.T, msg, gmx, spxcode, gmxfile, spxfile string)
 		t.Fatal("ParseFSDir:", err)
 	}
 	conf := *gblConf
+	conf.RelativeBase = "/foo"
 	conf.Recorder = nil
 	conf.NoFileLine = false
-	conf.WorkingDir = "/foo"
-	conf.TargetDir = "/foo"
 	bar := pkgs["main"]
 	_, err = cl.NewPackage("", bar, &conf)
 	if err == nil {
@@ -123,8 +136,8 @@ func gopSpxErrorTestEx(t *testing.T, msg, gmx, spxcode, gmxfile, spxfile string)
 }
 
 func TestSpxError(t *testing.T) {
-	gopSpxErrorTestEx(t, `./Game.tgmx:6:2: userScore redeclared
-	./Game.tgmx:5:2 other declaration of userScore`, `
+	gopSpxErrorTestEx(t, `Game.tgmx:6:2: userScore redeclared
+	Game.tgmx:5:2 other declaration of userScore`, `
 import "bytes"
 var (
 	Kai Kai
@@ -135,8 +148,8 @@ var (
 println "hi"
 `, "Game.tgmx", "Kai.tspx")
 
-	gopSpxErrorTestEx(t, `./Kai.tspx:4:2: id redeclared
-	./Kai.tspx:3:2 other declaration of id`, `
+	gopSpxErrorTestEx(t, `Kai.tspx:4:2: id redeclared
+	Kai.tspx:3:2 other declaration of id`, `
 var (
 	Kai Kai
 	userScore int
@@ -156,6 +169,8 @@ func onInit() {
 	for {
 	}
 }
+
+initGameApp
 `, `
 func onMsg(msg string) {
 	for {
@@ -169,18 +184,22 @@ import "github.com/goplus/gop/cl/internal/spx"
 type Game struct {
 	*spx.MyGame
 }
+type Kai struct {
+	spx.Sprite
+	*Game
+}
 
 func (this *Game) onInit() {
 	for {
 		spx.SchedNow()
 	}
 }
-
-type Kai struct {
-	spx.Sprite
-	*Game
+func (this *Game) MainEntry() {
+	this.InitGameApp()
+}
+func main() {
+	spx.Gopt_MyGame_Main(new(Game))
 }
-
 func (this *Kai) onMsg(msg string) {
 	for {
 		spx.Sched()
@@ -217,6 +236,10 @@ import (
 
 const Foo = 1
 
+type bar struct {
+	spx.Sprite
+	*index
+}
 type index struct {
 	*spx.MyGame
 }
@@ -255,6 +278,10 @@ import (
 type Game struct {
 	*spx.MyGame
 }
+type bar struct {
+	spx.Sprite
+	*Game
+}
 
 func (this *Game) onInit() {
 	spx.Sched()
@@ -262,12 +289,6 @@ func (this *Game) onInit() {
 	spx.TestIntValue = 1
 	x := math.Round(1.2)
 }
-
-type bar struct {
-	spx.Sprite
-	*Game
-}
-
 func (this *bar) onInit() {
 	this.SetCostume("kai-a")
 	this.Play("recordingWhere")
@@ -303,15 +324,15 @@ func onCloned() {
 
 import "github.com/goplus/gop/cl/internal/spx"
 
+type Game struct {
+	*spx.MyGame
+	Kai Kai
+}
 type Kai struct {
 	spx.Sprite
 	*Game
 	a int
 }
-type Game struct {
-	*spx.MyGame
-	Kai Kai
-}
 
 func (this *Game) onInit() {
 	spx.Gopt_Sprite_Clone__0(this.Kai)
@@ -349,15 +370,14 @@ type Kai struct {
 	spx.Sprite
 	*index
 }
-
-var x float64 = spx.Rand__1(1.2)
-
 type index struct {
 	*spx.MyGame
 	Kai Kai
 	t   spx.Sound
 }
 
+var x float64 = spx.Rand__1(1.2)
+
 func (this *index) MainEntry() {
 	spx.Gopt_MyGame_Run(this, "hzip://open.qiniu.us/weather/res.zip")
 }
@@ -370,6 +390,77 @@ func (this *Kai) Main() {
 `, "index.tgmx", "Kai.tspx")
 }
 
+func TestSpxRunWithWorkers(t *testing.T) {
+	gopSpxTestEx(t, `
+var (
+	Kai Kai
+)
+
+run
+`, `
+echo jwt.token("Hi")
+`, `package main
+
+import (
+	"fmt"
+	"github.com/goplus/gop/cl/internal/spx3"
+	"github.com/goplus/gop/cl/internal/spx3/jwt"
+)
+
+type Kai struct {
+	spx3.Sprite
+	*Game
+}
+type Game struct {
+	spx3.Game
+	Kai Kai
+}
+
+func (this *Game) MainEntry() {
+	this.Run()
+}
+func main() {
+	spx3.Gopt_Game_Main(new(Game), new(Kai))
+}
+func (this *Kai) Main() {
+	fmt.Println(jwt.Token("Hi"))
+}
+`, "main_spx.gox", "Kai_spx.gox")
+}
+
+func TestSpxNewObj(t *testing.T) {
+	gopSpxTestEx(t, `
+a := new
+a.run
+b := new(Sprite)
+echo b.name
+`, ``, `package main
+
+import (
+	"fmt"
+	"github.com/goplus/gop/cl/internal/spx3"
+)
+
+type Kai struct {
+	spx3.Sprite
+	*Game
+}
+type Game struct {
+	spx3.Game
+}
+
+func (this *Game) MainEntry() {
+	a := spx3.New()
+	a.Run()
+	b := new(spx3.Sprite)
+	fmt.Println(b.Name())
+}
+func main() {
+	spx3.Gopt_Game_Main(new(Game), new(Kai))
+}
+`, "main_spx.gox", "Kai_spx.gox")
+}
+
 func TestSpx2(t *testing.T) {
 	gopSpxTestEx(t, `
 println("Hi")
@@ -386,6 +477,10 @@ import (
 type Game struct {
 	spx2.Game
 }
+type Kai struct {
+	spx2.Sprite
+	*Game
+}
 
 func (this *Game) MainEntry() {
 	fmt.Println("Hi")
@@ -393,16 +488,12 @@ func (this *Game) MainEntry() {
 func main() {
 	new(Game).Main()
 }
-
-type Kai struct {
-	spx2.Sprite
-	*Game
-}
-
 func (this *Kai) onMsg(msg string) {
 }
 `, "Game.t2gmx", "Kai.t2spx")
+}
 
+func TestSpx3(t *testing.T) {
 	gopSpxTestEx(t, `
 println("Hi, Sprite2")
 `, `
@@ -418,6 +509,10 @@ import (
 type Game struct {
 	spx2.Game
 }
+type Kai struct {
+	spx2.Sprite2
+	*Game
+}
 
 func (this *Game) MainEntry() {
 	fmt.Println("Hi, Sprite2")
@@ -425,16 +520,12 @@ func (this *Game) MainEntry() {
 func main() {
 	new(Game).Main()
 }
-
-type Kai struct {
-	spx2.Sprite2
-	*Game
-}
-
 func (this *Kai) onMsg(msg string) {
 }
 `, "Game.t2gmx", "Kai.t2spx2")
+}
 
+func TestSpx4(t *testing.T) {
 	gopSpxTestEx(t, `
 println("Hi, Sprite")
 `, `
@@ -450,18 +541,16 @@ import (
 type Dog struct {
 	spx2.Sprite
 }
-
-func (this *Dog) Main() {
-	fmt.Println("Hi, Sprite")
-}
-
 type Kai struct {
 	spx2.Sprite2
 }
 
+func (this *Dog) Main() {
+	fmt.Println("Hi, Sprite")
+}
 func (this *Kai) onMsg(msg string) {
 }
-`, "Dog.t3spx", "Kai.t3spx2")
+`, "Dog_t3spx.gox", "Kai.t3spx2")
 }
 
 func TestSpxMainEntry(t *testing.T) {
@@ -478,13 +567,17 @@ import "github.com/goplus/gop/cl/internal/spx2"
 type Game struct {
 	spx2.Game
 }
+type Kai struct {
+	spx2.Sprite
+	*Game
+}
 
 func (this *Game) MainEntry() {
 }
 func main() {
 	new(Game).Main()
 }
-`, "Game.t2gmx", "Kai.t2spx")
+`, "Game.t2gmx", "Kai.t2spx", "")
 	gopSpxTestExConf(t, "OnlyGmx", &conf, `
 var (
 	Kai Kai
@@ -494,21 +587,21 @@ var (
 
 import "github.com/goplus/gop/cl/internal/spx2"
 
-type Kai struct {
-	spx2.Sprite
-	*Game
-}
 type Game struct {
 	spx2.Game
 	Kai Kai
 }
+type Kai struct {
+	spx2.Sprite
+	*Game
+}
 
 func (this *Game) MainEntry() {
 }
 func main() {
 	new(Game).Main()
 }
-`, "Game.t2gmx", "Kai.t2spx")
+`, "Game.t2gmx", "Kai.t2spx", "")
 
 	gopSpxTestExConf(t, "KaiAndGmx", &conf, `
 var (
@@ -530,14 +623,14 @@ import (
 	"github.com/goplus/gop/cl/internal/spx2"
 )
 
-type Kai struct {
-	spx2.Sprite
-	*Game
-}
 type Game struct {
 	spx2.Game
 	Kai Kai
 }
+type Kai struct {
+	spx2.Sprite
+	*Game
+}
 
 func (this *Game) MainEntry() {
 	fmt.Println("Hi")
@@ -550,7 +643,7 @@ func (this *Kai) Main() {
 }
 func (this *Kai) onMsg(msg string) {
 }
-`, "Game.t2gmx", "Kai.t2spx")
+`, "Game.t2gmx", "Kai.t2spx", "")
 }
 
 func TestSpxGoxBasic(t *testing.T) {
@@ -572,25 +665,23 @@ import "github.com/goplus/gop/cl/internal/spx"
 type Game struct {
 	*spx.MyGame
 }
+type Kai struct {
+	spx.Sprite
+	*Game
+}
 
 func (this *Game) onInit() {
 	for {
 		spx.SchedNow()
 	}
 }
-
-type Kai struct {
-	spx.Sprite
-	*Game
-}
-
 func (this *Kai) onMsg(msg string) {
 	for {
 		spx.Sched()
 		this.Say("Hi")
 	}
 }
-`, "Game.tgmx.gox", "Kai.tspx.gox")
+`, "Game.tgmx", "Kai.tspx")
 }
 
 func TestSpxClone(t *testing.T) {
@@ -631,15 +722,15 @@ type info struct {
 	x int
 	y int
 }
+type Game struct {
+	*spx.MyGame
+	Kai Kai
+}
 type Kai struct {
 	spx.Sprite
 	*Game
 	a int
 }
-type Game struct {
-	*spx.MyGame
-	Kai Kai
-}
 
 func (this *Game) onInit() {
 	spx.Gopt_Sprite_Clone__0(this.Kai)
@@ -658,7 +749,7 @@ func (this *Kai) onCloned() {
 }
 
 func TestSpxErrorSel(t *testing.T) {
-	gopSpxErrorTestEx(t, `./Kai.tspx:2:9: this.pos undefined (type *Kai has no field or method pos)`, `
+	gopSpxErrorTestEx(t, `Kai.tspx:2:9: this.pos undefined (type *Kai has no field or method pos)`, `
 println "hi"
 `, `
 println this.pos
@@ -678,6 +769,10 @@ import "github.com/goplus/gop/cl/internal/spx"
 type Game struct {
 	*spx.MyGame
 }
+type Kai struct {
+	spx.Sprite
+	*Game
+}
 
 func (this *Game) MainEntry() {
 	this.SendMessage("Hi")
@@ -685,12 +780,6 @@ func (this *Game) MainEntry() {
 func main() {
 	spx.Gopt_MyGame_Main(new(Game))
 }
-
-type Kai struct {
-	spx.Sprite
-	*Game
-}
-
 func (this *Kai) onMsg(msg string) {
 }
 `, "Game.tgmx", "Kai.tspx")
@@ -713,6 +802,10 @@ import (
 type Game struct {
 	*spx.MyGame
 }
+type Kai struct {
+	spx.Sprite
+	*Game
+}
 
 func (this *Game) MainEntry() {
 	fmt.Println("Hi")
@@ -720,12 +813,6 @@ func (this *Game) MainEntry() {
 func main() {
 	spx.Gopt_MyGame_Main(new(Game))
 }
-
-type Kai struct {
-	spx.Sprite
-	*Game
-}
-
 func (this *Kai) onMsg(msg string) {
 	this.Position().Add__0(100, 200)
 }
@@ -762,6 +849,10 @@ import (
 type Game struct {
 	*spx.MyGame
 }
+type Kai struct {
+	spx.Sprite
+	*Game
+}
 
 func (this *Game) MainEntry() {
 	fmt.Println("hi")
@@ -769,12 +860,6 @@ func (this *Game) MainEntry() {
 func main() {
 	spx.Gopt_MyGame_Main(new(Game))
 }
-
-type Kai struct {
-	spx.Sprite
-	*Game
-}
-
 func (this *Kai) onMsg(msg string) {
 	fmt.Println(msg)
 	this.Position().Add__0(100, 200)
@@ -792,3 +877,316 @@ func (this *Kai) onMsg(msg string) {
 }
 `, "Game.tgmx", "Kai.tspx")
 }
+
+func TestSpxOverload(t *testing.T) {
+	gopSpxTestEx(t, `
+var (
+	Kai Kai
+)
+
+func onInit() {
+	Kai.onKey "hello", key => {
+	}
+}
+`, `
+var (
+	a int
+)
+
+type Mesh struct {
+}
+
+func (p *Mesh) Name() string {
+	return "hello"
+}
+
+var (
+	m1 = &Mesh{}
+	m2 = &Mesh{}
+)
+
+onKey "hello", => {
+}
+onKey "hello", key => {
+}
+onKey ["1"], => {
+}
+onKey ["2"], key => {
+}
+onKey [m1, m2], => {
+}
+onKey [m1, m2], key => {
+}
+onKey ["a"], ["b"], key => {
+}
+onKey ["a"], [m1, m2], key => {
+}
+onKey ["a"], nil, key => {
+}
+onKey 100, 200
+onKey2 "hello", key => {
+}
+`, `package main
+
+import "github.com/goplus/gop/cl/internal/spx"
+
+type Mesh struct {
+}
+type Game struct {
+	*spx.MyGame
+	Kai Kai
+}
+type Kai struct {
+	spx.Sprite
+	*Game
+	a int
+}
+
+func (this *Game) onInit() {
+	spx.Gopt_Sprite_OnKey__1(this.Kai, "hello", func(key string) {
+	})
+}
+func (p *Mesh) Name() string {
+	return "hello"
+}
+
+var m1 = &Mesh{}
+var m2 = &Mesh{}
+
+func (this *Kai) Main() {
+	spx.Gopt_Sprite_OnKey__0(this, "hello", func() {
+	})
+	spx.Gopt_Sprite_OnKey__1(this, "hello", func(key string) {
+	})
+	spx.Gopt_Sprite_OnKey__2(this, []string{"1"}, func() {
+	})
+	spx.Gopt_Sprite_OnKey__3(this, []string{"2"}, func(key string) {
+	})
+	spx.Gopt_Sprite_OnKey__4(this, []spx.Mesher{m1, m2}, func() {
+	})
+	spx.Gopt_Sprite_OnKey__5(this, []spx.Mesher{m1, m2}, func(key spx.Mesher) {
+	})
+	spx.Gopt_Sprite_OnKey__6(this, []string{"a"}, []string{"b"}, func(key string) {
+	})
+	spx.Gopt_Sprite_OnKey__7(this, []string{"a"}, []spx.Mesher{m1, m2}, func(key string) {
+	})
+	spx.Gopt_Sprite_OnKey__6(this, []string{"a"}, nil, func(key string) {
+	})
+	spx.Gopt_Sprite_OnKey__8(this, 100, 200)
+	spx.Gopt_Sprite_OnKey2(this, "hello", func(key string) {
+	})
+}
+`, "Game.tgmx", "Kai.tspx")
+}
+
+func TestTestClassFile(t *testing.T) {
+	gopSpxTestEx2(t, `
+println "Hi"
+`, `
+t.log "Hi"
+t.run "a test", t => {
+	t.fatal "failed"
+}
+`, `package main
+
+import (
+	"fmt"
+	"github.com/goplus/gop/test"
+	"testing"
+)
+
+type caseFoo struct {
+	test.Case
+	*App
+}
+type App struct {
+	test.App
+}
+
+func (this *App) MainEntry() {
+	fmt.Println("Hi")
+}
+func (this *caseFoo) Main() {
+	this.T().Log("Hi")
+	this.T().Run("a test", func(t *testing.T) {
+		t.Fatal("failed")
+	})
+}
+func TestFoo(t *testing.T) {
+	test.Gopt_Case_TestMain(new(caseFoo), t)
+}
+func TestMain(m *testing.M) {
+	test.Gopt_App_TestMain(new(App), m)
+}
+`, "main_xtest.gox", "Foo_xtest.gox", "_test")
+}
+
+func TestTestClassFile2(t *testing.T) {
+	gopSpxTestEx2(t, `
+println "Hi"
+`, `
+t.log "Hi"
+`, `package main
+
+import (
+	"github.com/goplus/gop/test"
+	"testing"
+)
+
+type case_foo struct {
+	test.Case
+}
+
+func (this *case_foo) Main() {
+	this.T().Log("Hi")
+}
+func Test_foo(t *testing.T) {
+	test.Gopt_Case_TestMain(new(case_foo), t)
+}
+`, "main.gox", "foo_xtest.gox", "_test")
+}
+
+func TestGopxNoFunc(t *testing.T) {
+	gopClTestFile(t, `
+var (
+	a int
+)
+`, `package main
+
+type foo struct {
+	a int
+}
+`, "foo.gox")
+}
+
+func TestClassFileGopx(t *testing.T) {
+	gopClTestFile(t, `
+var (
+	BaseClass
+	Width, Height float64
+	*AggClass
+)
+
+type BaseClass struct{
+	x int
+	y int
+}
+type AggClass struct{}
+
+func Area() float64 {
+	return Width * Height
+}
+`, `package main
+
+type BaseClass struct {
+	x int
+	y int
+}
+type AggClass struct {
+}
+type Rect struct {
+	BaseClass
+	Width  float64
+	Height float64
+	*AggClass
+}
+
+func (this *Rect) Area() float64 {
+	return this.Width * this.Height
+}
+`, "Rect.gox")
+	gopClTestFile(t, `
+import "bytes"
+var (
+	bytes.Buffer
+)
+func test(){}
+`, `package main
+
+import "bytes"
+
+type Rect struct {
+	bytes.Buffer
+}
+
+func (this *Rect) test() {
+}
+`, "Rect.gox")
+	gopClTestFile(t, `
+import "bytes"
+var (
+	*bytes.Buffer
+)
+func test(){}
+`, `package main
+
+import "bytes"
+
+type Rect struct {
+	*bytes.Buffer
+}
+
+func (this *Rect) test() {
+}
+`, "Rect.gox")
+	gopClTestFile(t, `
+import "bytes"
+var (
+	*bytes.Buffer "spec:\"buffer\""
+	a int "json:\"a\""
+	b int
+)
+func test(){}
+`, `package main
+
+import "bytes"
+
+type Rect struct {
+	*bytes.Buffer `+"`spec:\"buffer\"`"+`
+	a             int `+"`json:\"a\"`"+`
+	b             int
+}
+
+func (this *Rect) test() {
+}
+`, "Rect.gox")
+}
+
+func TestClassFileMember(t *testing.T) {
+	gopClTestFile(t, `type Engine struct {
+}
+
+func (e *Engine) EnterPointerLock() {
+}
+
+func (e *Engine) SetEnable(b bool) {
+}
+
+func Engine() *Engine {
+	return &Engine{}
+}
+
+func Test() {
+	engine.setEnable true
+	engine.enterPointerLock
+}
+`, `package main
+
+type Engine struct {
+}
+type Rect struct {
+}
+
+func (e *Engine) EnterPointerLock() {
+}
+func (e *Engine) SetEnable(b bool) {
+}
+func (this *Rect) Engine() *Engine {
+	return &Engine{}
+}
+func (this *Rect) Test() {
+	this.Engine().SetEnable(true)
+	this.Engine().EnterPointerLock()
+}
+`, "Rect.gox")
+}
