@@ -19,7 +19,6 @@ package cl_test
 import (
 	"bytes"
 	"os"
-	"runtime"
 	"strings"
 	"sync"
 	"testing"
@@ -46,7 +45,7 @@ var (
 
 func init() {
 	gox.SetDebug(gox.DbgFlagAll)
-	cl.SetDebug(cl.DbgFlagAll)
+	cl.SetDebug(cl.DbgFlagAll | cl.FlagNoMarkAutogen)
 	gblFset = token.NewFileSet()
 	imp := gop.NewImporter(nil, &env.Gop{Root: gopRootDir, Version: "1.0"}, gblFset)
 	gblConf = &cl.Config{
@@ -68,8 +67,6 @@ func init() {
 		C2goBase:      "github.com/goplus/gop/cl/internal",
 		NoFileLine:    false,
 		NoAutoGenMain: true,
-		RelativePath:  true,
-		TargetDir:     ".",
 	}
 }
 
@@ -159,184 +156,6 @@ ls
 `)
 }
 
-func TestFileOpen(t *testing.T) {
-	gopClTest(t, `
-for line <- open("foo.txt")! {
-	println line
-}
-`, `package main
-
-import (
-	"fmt"
-	"os"
-	"github.com/goplus/gop/builtin/iox"
-	"github.com/qiniu/x/errors"
-)
-
-func main() {
-	for _gop_it := iox.EnumLines(func() (_gop_ret *os.File) {
-		var _gop_err error
-		_gop_ret, _gop_err = os.Open("foo.txt")
-		if _gop_err != nil {
-			_gop_err = errors.NewFrame(_gop_err, "open(\"foo.txt\")", "/foo/bar.gop", 2, "main.main")
-			panic(_gop_err)
-		}
-		return
-	}()); ; {
-		var _gop_ok bool
-		line, _gop_ok := _gop_it.Next()
-		if !_gop_ok {
-			break
-		}
-		fmt.Println(line)
-	}
-}
-`)
-}
-
-func TestFileEnumLines(t *testing.T) {
-	gopClTest(t, `
-import "os"
-
-for line <- os.Stdin {
-	println line
-}
-`, `package main
-
-import (
-	"fmt"
-	"os"
-	"github.com/goplus/gop/builtin/iox"
-)
-
-func main() {
-	for _gop_it := iox.EnumLines(os.Stdin); ; {
-		var _gop_ok bool
-		line, _gop_ok := _gop_it.Next()
-		if !_gop_ok {
-			break
-		}
-		fmt.Println(line)
-	}
-}
-`)
-}
-
-func TestIoxLines(t *testing.T) {
-	gopClTest(t, `
-import "io"
-
-var r io.Reader
-
-for line <- lines(r) {
-	println line
-}
-`, `package main
-
-import (
-	"fmt"
-	"github.com/goplus/gop/builtin/iox"
-	"io"
-)
-
-var r io.Reader
-
-func main() {
-	for _gop_it := iox.Lines(r).Gop_Enum(); ; {
-		var _gop_ok bool
-		line, _gop_ok := _gop_it.Next()
-		if !_gop_ok {
-			break
-		}
-		fmt.Println(line)
-	}
-}
-`)
-}
-
-func TestMixedGo(t *testing.T) {
-	gopMixedClTest(t, "main", `package main
-
-import "strconv"
-
-const n = 10
-
-func f(v int) string {
-	return strconv.Itoa(v)
-}
-
-type foo struct {
-	v int
-}
-
-func (a foo) _() {
-}
-
-func (a foo) Str() string {
-	return f(a.v)
-}
-
-func (a *foo) Bar() int {
-	return 0
-}
-
-type foo2 = foo
-type foo3 foo2
-`, `
-var a [n]int
-var b string = f(n)
-var c foo2
-var d int = c.v
-var e = foo3{}
-var x string = c.str
-`, `package main
-
-var a [10]int
-var b string = f(n)
-var c foo
-var d int = c.v
-var e = foo3{}
-var x string = c.Str()
-`, true)
-	gopMixedClTest(t, "main", `package main
-type Point struct {
-	X int
-	Y int
-}
-`, `
-type T struct{}
-println(&T{},&Point{10,20})
-`, `package main
-
-import "fmt"
-
-type T struct {
-}
-
-func main() {
-	fmt.Println(&T{}, &Point{10, 20})
-}
-`, false)
-}
-
-func Test_RangeExpressionIf_Issue1243(t *testing.T) {
-	gopClTest(t, `
-for i <- :10, i%3 == 0 {
-	println i
-}`, `package main
-
-import "fmt"
-
-func main() {
-	for i := 0; i < 10; i += 1 {
-		if i%3 == 0 {
-			fmt.Println(i)
-		}
-	}
-}
-`)
-}
-
 func Test_CastSlice_Issue1240(t *testing.T) {
 	gopClTest(t, `
 type fvec []float64
@@ -1477,9 +1296,9 @@ func foo(script string) {
 
 import (
 	"fmt"
+	"github.com/goplus/gop/ast/gopq"
 	"github.com/goplus/gop/ast/goptest"
 	"github.com/qiniu/x/errors"
-	"github.com/goplus/gop/ast/gopq"
 )
 
 func foo(script string) {
@@ -1511,9 +1330,9 @@ func foo(script string) {
 
 import (
 	"fmt"
+	"github.com/goplus/gop/ast/gopq"
 	"github.com/goplus/gop/ast/goptest"
 	"github.com/qiniu/x/errors"
-	"github.com/goplus/gop/ast/gopq"
 )
 
 func foo(script string) {
@@ -1542,8 +1361,8 @@ func add(x, y string) (int, error) {
 `, `package main
 
 import (
-	"strconv"
 	"github.com/qiniu/x/errors"
+	"strconv"
 )
 
 func add(x string, y string) (int, error) {
@@ -1716,7 +1535,7 @@ var z uint128 = x + y
 import "github.com/goplus/gop/builtin/ng"
 
 var x, y ng.Uint128
-var z ng.Uint128 = x.Gop_Add__1(y)
+var z ng.Uint128 = (ng.Uint128).Gop_Add__1(x, y)
 `)
 }
 
@@ -1729,7 +1548,7 @@ var z int128 = x + y
 import "github.com/goplus/gop/builtin/ng"
 
 var x, y ng.Int128
-var z ng.Int128 = x.Gop_Add__1(y)
+var z ng.Int128 = (ng.Int128).Gop_Add__1(x, y)
 `)
 }
 
@@ -1742,7 +1561,7 @@ var z bigint = x + y
 import "github.com/goplus/gop/builtin/ng"
 
 var x, y ng.Bigint
-var z ng.Bigint = x.Gop_Add(y)
+var z ng.Bigint = (ng.Bigint).Gop_Add(x, y)
 `)
 }
 
@@ -1824,8 +1643,8 @@ import (
 )
 
 var x = ng.Bigrat_Init__2(big.NewRat(7, 2))
-var y = x.Gop_Add(ng.Bigrat_Init__0(100))
-var z = ng.Bigrat_Init__0(100) + y
+var y = (ng.Bigrat).Gop_Add(x, ng.Bigrat_Init__0(100))
+var z = (ng.Bigrat).Gop_Add(ng.Bigrat_Init__0(100), y)
 `)
 }
 
@@ -2939,71 +2758,6 @@ func (M) Bar() {
 `)
 }
 
-func TestOverloadOp(t *testing.T) {
-	gopClTest(t, `
-type foo struct {
-}
-
-func (a *foo) + (b *foo) *foo {
-	println("a + b")
-	return &foo{}
-}
-
-func (a foo) - (b foo) foo {
-	println("a - b")
-	return foo{}
-}
-
-func -(a foo) {
-	println("-a")
-}
-
-func ++(a foo) {
-	println("a++")
-}
-
-func (a foo) != (b foo) bool{
-	println("a!=b")	
-	return true
-}
-
-var a, b foo
-var c = a - b
-var d = -a       // TODO: -a have no return value!
-var e = a!=b
-`, `package main
-
-import "fmt"
-
-type foo struct {
-}
-
-func (a *foo) Gop_Add(b *foo) *foo {
-	fmt.Println("a + b")
-	return &foo{}
-}
-func (a foo) Gop_Sub(b foo) foo {
-	fmt.Println("a - b")
-	return foo{}
-}
-func (a foo) Gop_NE(b foo) bool {
-	fmt.Println("a!=b")
-	return true
-}
-func (a foo) Gop_Neg() {
-	fmt.Println("-a")
-}
-func (a foo) Gop_Inc() {
-	fmt.Println("a++")
-}
-
-var a, b foo
-var c = a.Gop_Sub(b)
-var d = a.Gop_Neg()
-var e = a.Gop_NE(b)
-`)
-}
-
 func TestCmdlineNoEOL(t *testing.T) {
 	gopClTest(t, `println "Hi"`, `package main
 
@@ -4359,250 +4113,6 @@ func main() {
 `)
 }
 
-func TestClassFileGopx(t *testing.T) {
-	gopClTestFile(t, `
-var (
-	BaseClass
-	Width, Height float64
-	*AggClass
-)
-
-type BaseClass struct{
-	x int
-	y int
-}
-type AggClass struct{}
-
-func Area() float64 {
-	return Width * Height
-}
-`, `package main
-
-type BaseClass struct {
-	x int
-	y int
-}
-type AggClass struct {
-}
-type Rect struct {
-	BaseClass
-	Width  float64
-	Height float64
-	*AggClass
-}
-
-func (this *Rect) Area() float64 {
-	return this.Width * this.Height
-}
-`, "Rect.gox")
-	gopClTestFile(t, `
-import "bytes"
-var (
-	bytes.Buffer
-)
-func test(){}
-`, `package main
-
-import "bytes"
-
-type Rect struct {
-	bytes.Buffer
-}
-
-func (this *Rect) test() {
-}
-`, "Rect.gox")
-	gopClTestFile(t, `
-import "bytes"
-var (
-	*bytes.Buffer
-)
-func test(){}
-`, `package main
-
-import "bytes"
-
-type Rect struct {
-	*bytes.Buffer
-}
-
-func (this *Rect) test() {
-}
-`, "Rect.gox")
-	gopClTestFile(t, `
-import "bytes"
-var (
-	*bytes.Buffer "spec:\"buffer\""
-	a int "json:\"a\""
-	b int
-)
-func test(){}
-`, `package main
-
-import "bytes"
-
-type Rect struct {
-	*bytes.Buffer `+"`spec:\"buffer\"`"+`
-	a             int `+"`json:\"a\"`"+`
-	b             int
-}
-
-func (this *Rect) test() {
-}
-`, "Rect.gox")
-}
-
-func TestOverload(t *testing.T) {
-	gopClTest(t, `
-import "github.com/goplus/gop/cl/internal/overload/foo"
-
-type Mesh struct {
-}
-
-func (p *Mesh) Name() string {
-	return "hello"
-}
-
-var (
-	m1 = &Mesh{}
-	m2 = &Mesh{}
-)
-
-foo.onKey "hello", => {
-}
-foo.onKey "hello", key => {
-}
-foo.onKey ["1"], => {
-}
-foo.onKey ["2"], key => {
-}
-foo.onKey [m1,m2], => {
-}
-foo.onKey [m1,m2], key => {
-}
-foo.onKey ["a"], ["b"], key => {
-}
-foo.onKey ["a"], [m1,m2], key => {
-}
-foo.onKey ["a"],nil,key => {
-}
-n := &foo.N{}
-n.onKey "hello", => {
-}
-n.onKey "hello", key => {
-}
-n.onKey ["1"], => {
-}
-n.onKey ["2"], key => {
-}
-n.onKey [m1,m2], => {
-}
-n.onKey [m1,m2], key => {
-}
-n.onKey ["a"], ["b"], key => {
-}
-n.onKey ["a"], [m1,m2], key => {
-}
-n.onKey ["a"],nil,key => {
-}
-`, `package main
-
-import "github.com/goplus/gop/cl/internal/overload/foo"
-
-type Mesh struct {
-}
-
-func (p *Mesh) Name() string {
-	return "hello"
-}
-
-var m1 = &Mesh{}
-var m2 = &Mesh{}
-
-func main() {
-	foo.OnKey__0("hello", func() {
-	})
-	foo.OnKey__1("hello", func(key string) {
-	})
-	foo.OnKey__2([]string{"1"}, func() {
-	})
-	foo.OnKey__3([]string{"2"}, func(key string) {
-	})
-	foo.OnKey__4([]foo.Mesher{m1, m2}, func() {
-	})
-	foo.OnKey__5([]foo.Mesher{m1, m2}, func(key foo.Mesher) {
-	})
-	foo.OnKey__6([]string{"a"}, []string{"b"}, func(key string) {
-	})
-	foo.OnKey__7([]string{"a"}, []foo.Mesher{m1, m2}, func(key string) {
-	})
-	foo.OnKey__6([]string{"a"}, nil, func(key string) {
-	})
-	n := &foo.N{}
-	n.OnKey__0("hello", func() {
-	})
-	n.OnKey__1("hello", func(key string) {
-	})
-	n.OnKey__2([]string{"1"}, func() {
-	})
-	n.OnKey__3([]string{"2"}, func(key string) {
-	})
-	n.OnKey__4([]foo.Mesher{m1, m2}, func() {
-	})
-	n.OnKey__5([]foo.Mesher{m1, m2}, func(key foo.Mesher) {
-	})
-	n.OnKey__6([]string{"a"}, []string{"b"}, func(key string) {
-	})
-	n.OnKey__7([]string{"a"}, []foo.Mesher{m1, m2}, func(key string) {
-	})
-	n.OnKey__6([]string{"a"}, nil, func(key string) {
-	})
-}
-`)
-}
-
-func TestClassFileMember(t *testing.T) {
-	gopClTestFile(t, `type Engine struct {
-}
-
-func (e *Engine) EnterPointerLock() {
-}
-
-func (e *Engine) SetEnable(b bool) {
-}
-
-
-func Engine() *Engine {
-	return &Engine{}
-}
-
-func Test() {
-	engine.setEnable true
-	engine.enterPointerLock
-}
-`, `package main
-
-type Engine struct {
-}
-
-func (e *Engine) EnterPointerLock() {
-}
-func (e *Engine) SetEnable(b bool) {
-}
-
-type Rect struct {
-}
-
-func (this *Rect) Engine() *Engine {
-	return &Engine{}
-}
-func (this *Rect) Test() {
-	this.Engine().SetEnable(true)
-	this.Engine().EnterPointerLock()
-}
-`, "Rect.gox")
-}
-
 func TestCommentLine(t *testing.T) {
 	gopClTestEx(t, gblConfLine, "main", `
 type Point struct {
@@ -4657,9 +4167,7 @@ func main() {
 func TestCommentLineRoot(t *testing.T) {
 	conf := *gblConf
 	conf.NoFileLine = false
-	conf.FileLineRoot = "/foo/root"
-	conf.WorkingDir = "/foo"
-	conf.TargetDir = "/foo"
+	conf.RelativeBase = "/foo/root"
 	var src = `
 type Point struct {
 	x int
@@ -4709,8 +4217,5 @@ func main() {
 	testPoint()
 }
 `
-	if runtime.GOOS == "windows" {
-		expected = strings.Replace(expected, "../", `../foo/`, -1)
-	}
 	gopClTestEx(t, &conf, "main", src, expected)
 }
