@@ -28,7 +28,6 @@ import (
 	"strings"
 
 	"github.com/goplus/gop/ast"
-	"github.com/goplus/gop/cl/internal/typesutil"
 	"github.com/goplus/gop/printer"
 	"github.com/goplus/gop/token"
 	"github.com/goplus/gox"
@@ -65,6 +64,7 @@ const (
 	clIdentGoto
 	clCallWithTwoValue
 	clCommandWithoutArgs
+	clMayBuiltinNew // maybe is builtin directive `new`
 )
 
 const (
@@ -75,7 +75,7 @@ const (
 
 const errorPkgPath = "github.com/qiniu/x/errors"
 
-func compileIdent(ctx *blockCtx, ident *ast.Ident, flags int) (pkg *gox.PkgRef, kind int) {
+func compileIdent(ctx *blockCtx, ident *ast.Ident, flags int) (pkg gox.PkgRef, kind int) {
 	fvalue := (flags&clIdentSelectorExpr) != 0 || (flags&clIdentLHS) == 0
 	name := ident.Name
 	if name == "_" {
@@ -100,7 +100,7 @@ func compileIdent(ctx *blockCtx, ident *ast.Ident, flags int) (pkg *gox.PkgRef,
 			sig := fn.Ancestor().Type().(*types.Signature)
 			if recv := sig.Recv(); recv != nil {
 				ctx.cb.Val(recv)
-				chkFlag := flags &^ clCommandWithoutArgs
+				chkFlag := flags // &^ clCommandWithoutArgs (TODO: why?)
 				if chkFlag&clIdentSelectorExpr != 0 {
 					chkFlag = clIdentCanAutoCall
 				}
@@ -123,7 +123,8 @@ func compileIdent(ctx *blockCtx, ident *ast.Ident, flags int) (pkg *gox.PkgRef,
 	// pkgRef object
 	if (flags & clIdentSelectorExpr) != 0 {
 		if name == "C" && len(ctx.clookups) > 0 {
-			return nil, objCPkgRef
+			kind = objCPkgRef
+			return
 		}
 		if pi, ok := ctx.findImport(name); ok {
 			if rec := ctx.recorder(); rec != nil {
@@ -134,7 +135,7 @@ func compileIdent(ctx *blockCtx, ident *ast.Ident, flags int) (pkg *gox.PkgRef,
 	}
 
 	// object from import . "xxx"
-	if compilePkgRef(ctx, nil, ident, flags, objPkgRef) {
+	if compilePkgRef(ctx, gox.PkgRef{}, ident, flags, objPkgRef) {
 		return
 	}
 
@@ -161,12 +162,10 @@ find:
 	if rec := ctx.recorder(); rec != nil {
 		e := ctx.cb.Get(-1)
 		if oldo != nil && gox.IsTypeEx(e.Type) {
-			rec.Use(ident, oldo)
-			rec.Type(ident, typesutil.NewTypeAndValueForObject(oldo))
+			rec.recordIdent(ctx, ident, oldo)
 			return
 		}
-		rec.Use(ident, o)
-		rec.Type(ident, typesutil.NewTypeAndValueForObject(o))
+		rec.recordIdent(ctx, ident, o)
 	}
 	return
 }
@@ -202,33 +201,20 @@ func compileExprLHS(ctx *blockCtx, expr ast.Expr) {
 		compileIndexExprLHS(ctx, v)
 	case *ast.SelectorExpr:
 		compileSelectorExprLHS(ctx, v)
-		recordTypesVariable(ctx, v, -1)
 	case *ast.StarExpr:
 		compileStarExprLHS(ctx, v)
 	default:
 		log.Panicln("compileExpr failed: unknown -", reflect.TypeOf(v))
 	}
+	if rec := ctx.recorder(); rec != nil {
+		rec.recordExpr(ctx, expr, true)
+	}
 }
 
 func twoValue(inFlags []int) bool {
 	return inFlags != nil && (inFlags[0]&clCallWithTwoValue) != 0
 }
 
-func recordTypesValue(ctx *blockCtx, expr ast.Expr, n int) {
-	if rec := ctx.recorder(); rec != nil {
-		e := ctx.cb.Get(n)
-		rec.Type(expr, typesutil.NewTypeAndValueForValue(e.Type, e.CVal))
-	}
-}
-
-func recordTypesVariable(ctx *blockCtx, expr ast.Expr, n int) {
-	if rec := ctx.recorder(); rec != nil {
-		e := ctx.cb.Get(n)
-		t, _ := gox.DerefType(e.Type)
-		rec.Type(expr, typesutil.NewTypeAndValueForVariable(t))
-	}
-}
-
 func compileExpr(ctx *blockCtx, expr ast.Expr, inFlags ...int) {
 	switch v := expr.(type) {
 	case *ast.Ident:
@@ -239,7 +225,6 @@ func compileExpr(ctx *blockCtx, expr ast.Expr, inFlags ...int) {
 		compileIdent(ctx, v, flags)
 	case *ast.BasicLit:
 		compileBasicLit(ctx, v)
-		recordTypesValue(ctx, v, -1)
 	case *ast.CallExpr:
 		flags := 0
 		if inFlags != nil {
@@ -254,10 +239,8 @@ func compileExpr(ctx *blockCtx, expr ast.Expr, inFlags ...int) {
 		compileSelectorExpr(ctx, v, flags)
 	case *ast.BinaryExpr:
 		compileBinaryExpr(ctx, v)
-		recordTypesValue(ctx, v, -1)
 	case *ast.UnaryExpr:
 		compileUnaryExpr(ctx, v, twoValue(inFlags))
-		recordTypesValue(ctx, v, -1)
 	case *ast.FuncLit:
 		compileFuncLit(ctx, v)
 	case *ast.CompositeLit:
@@ -301,6 +284,9 @@ func compileExpr(ctx *blockCtx, expr ast.Expr, inFlags ...int) {
 	default:
 		log.Panicln("compileExpr failed: unknown -", reflect.TypeOf(v))
 	}
+	if rec := ctx.recorder(); rec != nil {
+		rec.recordExpr(ctx, expr, false)
+	}
 }
 
 func compileExprOrNone(ctx *blockCtx, expr ast.Expr) {
@@ -399,17 +385,13 @@ func compileSelectorExpr(ctx *blockCtx, v *ast.SelectorExpr, flags int) {
 		}
 	default:
 		compileExpr(ctx, v.X)
-		if rec := ctx.recorder(); rec != nil {
-			e := ctx.cb.Get(-1)
-			rec.Type(v.X, typesutil.NewTypeAndValueForType(e.Type))
-		}
 	}
 	if err := compileMember(ctx, v, v.Sel.Name, flags); err != nil {
 		panic(err)
 	}
 }
 
-func pkgRef(at *gox.PkgRef, name string) (o types.Object, alias bool) {
+func pkgRef(at gox.PkgRef, name string) (o types.Object, alias bool) {
 	if c := name[0]; c >= 'a' && c <= 'z' {
 		name = string(rune(c)+('A'-'a')) + name[1:]
 		if v := at.TryRef(name); v != nil && gox.IsFunc(v.Type()) {
@@ -420,8 +402,9 @@ func pkgRef(at *gox.PkgRef, name string) (o types.Object, alias bool) {
 	return at.TryRef(name), false
 }
 
-func lookupPkgRef(ctx *blockCtx, pkg *gox.PkgRef, x *ast.Ident, pkgKind int) (o types.Object, alias bool) {
-	if pkg != nil {
+// allow pkg.Types to be nil
+func lookupPkgRef(ctx *blockCtx, pkg gox.PkgRef, x *ast.Ident, pkgKind int) (o types.Object, alias bool) {
+	if pkg.Types != nil {
 		return pkgRef(pkg, x.Name)
 	}
 	if pkgKind == objPkgRef {
@@ -436,7 +419,7 @@ func lookupPkgRef(ctx *blockCtx, pkg *gox.PkgRef, x *ast.Ident, pkgKind int) (o
 			}
 		}
 	} else {
-		var cpkg *cpackages.PkgRef
+		var cpkg cpackages.PkgRef
 		for _, at := range ctx.clookups {
 			if o2 := at.Lookup(x.Name); o2 != nil {
 				if o != nil {
@@ -451,24 +434,28 @@ func lookupPkgRef(ctx *blockCtx, pkg *gox.PkgRef, x *ast.Ident, pkgKind int) (o
 	return
 }
 
-func compilePkgRef(ctx *blockCtx, at *gox.PkgRef, x *ast.Ident, flags, pkgKind int) bool {
+// allow at.Types to be nil
+func compilePkgRef(ctx *blockCtx, at gox.PkgRef, x *ast.Ident, flags, pkgKind int) bool {
 	if v, alias := lookupPkgRef(ctx, at, x, pkgKind); v != nil {
 		cb := ctx.cb
 		if (flags & clIdentLHS) != 0 {
+			if rec := ctx.recorder(); rec != nil {
+				rec.Use(x, v)
+			}
 			cb.VarRef(v, x)
 		} else {
 			autoprop := alias && (flags&clIdentCanAutoCall) != 0
 			if autoprop && !gox.HasAutoProperty(v.Type()) {
 				return false
 			}
+			if rec := ctx.recorder(); rec != nil {
+				rec.Use(x, v)
+			}
 			cb.Val(v, x)
 			if autoprop {
-				cb.Call(0)
+				cb.CallWith(0, 0, x)
 			}
 		}
-		if rec := ctx.recorder(); rec != nil {
-			rec.Use(x, v)
-		}
 		return true
 	}
 	return false
@@ -477,18 +464,18 @@ func compilePkgRef(ctx *blockCtx, at *gox.PkgRef, x *ast.Ident, flags, pkgKind i
 type fnType struct {
 	next     *fnType
 	params   *types.Tuple
-	n1       int
+	base     int
+	size     int
 	variadic bool
 	typetype bool
-	inited   bool
 }
 
 func (p *fnType) arg(i int, ellipsis bool) types.Type {
-	if i < p.n1 {
-		return p.params.At(i).Type()
+	if i+p.base < p.size {
+		return p.params.At(i + p.base).Type()
 	}
 	if p.variadic {
-		t := p.params.At(p.n1).Type()
+		t := p.params.At(p.size).Type()
 		if ellipsis {
 			return t
 		}
@@ -497,42 +484,61 @@ func (p *fnType) arg(i int, ellipsis bool) types.Type {
 	return nil
 }
 
-func (p *fnType) init(t *types.Signature) {
+func (p *fnType) init(base int, t *types.Signature) {
+	p.base = base
 	p.params, p.variadic = t.Params(), t.Variadic()
-	p.n1 = p.params.Len()
+	p.size = p.params.Len()
 	if p.variadic {
-		p.n1--
+		p.size--
 	}
 }
 
 func (p *fnType) initTypeType(t *gox.TypeType) {
 	param := types.NewParam(0, nil, "", t.Type())
 	p.params, p.typetype = types.NewTuple(param), true
-	p.n1 = 1
+	p.size = 1
 }
 
-func (p *fnType) initWith(fnt types.Type, idx, nin int) {
-	if p.inited {
-		return
+func (p *fnType) load(fnt types.Type) {
+	switch v := fnt.(type) {
+	case *gox.TypeType:
+		p.initTypeType(v)
+	case *types.Signature:
+		typ, objs := gox.CheckSigFuncExObjects(v)
+		switch typ.(type) {
+		case *gox.TyOverloadFunc, *gox.TyOverloadMethod:
+			p.initFuncs(0, objs)
+			return
+		case *gox.TyTemplateRecvMethod:
+			p.initFuncs(1, objs)
+			return
+		}
+		p.init(0, v)
 	}
-	p.inited = true
-	if t, ok := fnt.(*gox.TypeType); ok {
-		p.initTypeType(t)
-	} else if t := gox.CheckSignatures(fnt, idx, nin); t != nil {
-		p.init(t[0])
-		for i := 1; i < len(t); i++ {
-			fn := &fnType{}
-			fn.inited = true
-			fn.init(t[i])
-			p.next = fn
-			p = p.next
+}
+
+func (p *fnType) initFuncs(base int, funcs []types.Object) {
+	for i, obj := range funcs {
+		if sig, ok := obj.Type().(*types.Signature); ok {
+			if i == 0 {
+				p.init(base, sig)
+			} else {
+				fn := &fnType{}
+				fn.init(base, sig)
+				p.next = fn
+				p = p.next
+			}
 		}
 	}
 }
 
 func compileCallExpr(ctx *blockCtx, v *ast.CallExpr, inFlags int) {
+	var flags gox.InstrFlags
 	switch fn := v.Fun.(type) {
 	case *ast.Ident:
+		if len(v.Args) == 1 && fn.Name == "new" { // maybe new(T)
+			flags = clMayBuiltinNew
+		}
 		compileIdent(ctx, fn, clIdentAllowBuiltin|inFlags)
 	case *ast.SelectorExpr:
 		compileSelectorExpr(ctx, fn, 0)
@@ -550,21 +556,18 @@ func compileCallExpr(ctx *blockCtx, v *ast.CallExpr, inFlags int) {
 		compileExpr(ctx, fn)
 	}
 	var fnt = ctx.cb.Get(-1).Type
-	var flags gox.InstrFlags
 	var ellipsis = v.Ellipsis != gotoken.NoPos
 	if ellipsis {
-		flags = gox.InstrFlagEllipsis
+		flags |= gox.InstrFlagEllipsis
 	}
 	if (inFlags & clCallWithTwoValue) != 0 {
 		flags |= gox.InstrFlagTwoValue
 	}
-	var fn *fnType = &fnType{}
+	fn := &fnType{}
+	fn.load(fnt)
 	for fn != nil {
-		err := compileCallArgs(fn, fnt, ctx, v, ellipsis, flags)
+		err := compileCallArgs(fn, ctx, v, ellipsis, flags)
 		if err == nil {
-			if rec := ctx.recorder(); rec != nil {
-				rec.Type(v, typesutil.NewTypeAndValueForCallResult(ctx.cb.Get(-1).Type))
-			}
 			break
 		}
 		if fn.next == nil {
@@ -572,10 +575,15 @@ func compileCallExpr(ctx *blockCtx, v *ast.CallExpr, inFlags int) {
 		}
 		fn = fn.next
 	}
+	if rec := ctx.recorder(); rec != nil {
+		rec.recordCallExpr(ctx, v, fnt)
+	}
 }
 
-func compileCallArgs(fn *fnType, fnt types.Type, ctx *blockCtx, v *ast.CallExpr, ellipsis bool, flags gox.InstrFlags) (err error) {
-	n := ctx.cb.InternalStack().Len()
+func compileCallArgs(fn *fnType, ctx *blockCtx, v *ast.CallExpr, ellipsis bool, flags gox.InstrFlags) (err error) {
+	cb := ctx.cb
+	stk := cb.InternalStack()
+	n := stk.Len()
 	defer func() {
 		if r := recover(); r != nil {
 			if e, ok := r.(error); ok {
@@ -584,24 +592,20 @@ func compileCallArgs(fn *fnType, fnt types.Type, ctx *blockCtx, v *ast.CallExpr,
 				src := ctx.LoadExpr(v)
 				err = ctx.newCodeErrorf(v.Pos(), "compile func %v error: %v", src, r)
 			}
-			ctx.cb.InternalStack().SetLen(n)
+			stk.SetLen(n)
 		}
 	}()
 	for i, arg := range v.Args {
 		switch expr := arg.(type) {
 		case *ast.LambdaExpr:
-			fn.initWith(fnt, i, len(expr.Lhs))
-			sig := checkLambdaFuncType(ctx, expr, fn.arg(i, true), clLambaArgument, v.Fun)
+			sig := checkLambdaFuncType(ctx, expr, fn.arg(i, ellipsis), clLambaArgument, v.Fun)
 			compileLambdaExpr(ctx, expr, sig)
 		case *ast.LambdaExpr2:
-			fn.initWith(fnt, i, len(expr.Lhs))
-			sig := checkLambdaFuncType(ctx, expr, fn.arg(i, true), clLambaArgument, v.Fun)
+			sig := checkLambdaFuncType(ctx, expr, fn.arg(i, ellipsis), clLambaArgument, v.Fun)
 			compileLambdaExpr2(ctx, expr, sig)
 		case *ast.CompositeLit:
-			fn.initWith(fnt, i, -1)
 			compileCompositeLit(ctx, expr, fn.arg(i, ellipsis), true)
 		case *ast.SliceLit:
-			fn.initWith(fnt, i, -2)
 			t := fn.arg(i, ellipsis)
 			switch t.(type) {
 			case *types.Slice:
@@ -614,7 +618,7 @@ func compileCallArgs(fn *fnType, fnt types.Type, ctx *blockCtx, v *ast.CallExpr,
 			}
 			typetype := fn.typetype && t != nil
 			if typetype {
-				ctx.cb.InternalStack().Pop()
+				stk.Pop()
 			}
 			compileSliceLit(ctx, expr, t)
 			if typetype {
@@ -624,10 +628,23 @@ func compileCallArgs(fn *fnType, fnt types.Type, ctx *blockCtx, v *ast.CallExpr,
 			compileExpr(ctx, arg)
 		}
 	}
-	ctx.cb.CallWith(len(v.Args), flags, v)
+	if (flags & clMayBuiltinNew) != 0 { // maybe new(T): see TestSpxNewObj
+		arg := stk.Get(-1)
+		if _, ok := arg.Type.(*gox.TypeType); ok && isNonInstr(stk.Get(-2).Type) {
+			stk.PopN(2)
+			cb.Val(ctx.pkg.Builtin().Ref("new"))
+			stk.Push(arg)
+		}
+	}
+	cb.CallWith(len(v.Args), flags, v)
 	return
 }
 
+func isNonInstr(t types.Type) bool {
+	_, ok := t.(*gox.TyInstruction)
+	return !ok
+}
+
 type clLambaFlag string
 
 const (
@@ -686,7 +703,11 @@ func makeLambdaParams(ctx *blockCtx, pos token.Pos, lhs []*ast.Ident, in *types.
 	}
 	params := make([]*types.Var, n)
 	for i, name := range lhs {
-		params[i] = pkg.NewParam(name.Pos(), name.Name, in.At(i).Type())
+		param := pkg.NewParam(name.Pos(), name.Name, in.At(i).Type())
+		params[i] = param
+		if rec := ctx.recorder(); rec != nil {
+			rec.Def(name, param)
+		}
 	}
 	return types.NewTuple(params...)
 }
@@ -708,6 +729,9 @@ func compileLambdaExpr(ctx *blockCtx, v *ast.LambdaExpr, sig *types.Signature) {
 	params := makeLambdaParams(ctx, v.Pos(), v.Lhs, sig.Params())
 	results := makeLambdaResults(pkg, sig.Results())
 	ctx.cb.NewClosure(params, results, false).BodyStart(pkg)
+	if len(v.Lhs) > 0 {
+		defNames(ctx, v.Lhs, ctx.cb.Scope())
+	}
 	for _, v := range v.Rhs {
 		compileExpr(ctx, v)
 	}
@@ -720,7 +744,12 @@ func compileLambdaExpr2(ctx *blockCtx, v *ast.LambdaExpr2, sig *types.Signature)
 	results := makeLambdaResults(pkg, sig.Results())
 	comments, once := ctx.cb.BackupComments()
 	fn := ctx.cb.NewClosure(params, results, false)
-	loadFuncBody(ctx, fn, v.Body, v)
+	cb := fn.BodyStart(ctx.pkg, v.Body)
+	if len(v.Lhs) > 0 {
+		defNames(ctx, v.Lhs, cb.Scope())
+	}
+	compileStmts(ctx, v.Body.List)
+	cb.End(v)
 	ctx.cb.SetComments(comments, once)
 }
 
@@ -728,6 +757,9 @@ func compileFuncLit(ctx *blockCtx, v *ast.FuncLit) {
 	cb := ctx.cb
 	comments, once := cb.BackupComments()
 	sig := toFuncType(ctx, v.Type, nil, nil)
+	if rec := ctx.recorder(); rec != nil {
+		rec.recordFuncLit(ctx, v, sig)
+	}
 	fn := cb.NewClosureWith(sig)
 	if body := v.Body; body != nil {
 		loadFuncBody(ctx, fn, body, v)
@@ -756,7 +788,46 @@ func compileBasicLit(ctx *blockCtx, v *ast.BasicLit) {
 		}
 		cb.Val(rune(0)).ArrayLit(typ, n+1).UnaryOp(gotoken.AND).Call(1).Call(1)
 	default:
-		cb.Val(&goast.BasicLit{Kind: gotoken.Token(v.Kind), Value: v.Value}, v)
+		if v.Extra == nil {
+			basicLit(cb, v)
+			return
+		}
+		compileStringLitEx(ctx, cb, v)
+	}
+}
+
+func basicLit(cb *gox.CodeBuilder, v *ast.BasicLit) {
+	cb.Val(&goast.BasicLit{Kind: gotoken.Token(v.Kind), Value: v.Value}, v)
+}
+
+func compileStringLitEx(ctx *blockCtx, cb *gox.CodeBuilder, lit *ast.BasicLit) {
+	pos := lit.ValuePos + 1
+	quote := lit.Value[:1]
+	notFirst := false
+	for _, part := range lit.Extra.Parts {
+		switch v := part.(type) {
+		case string: // normal string literal or end with "$$"
+			next := pos + token.Pos(len(v))
+			if strings.HasSuffix(v, "$$") {
+				v = v[:len(v)-1]
+			}
+			basicLit(cb, &ast.BasicLit{ValuePos: pos - 1, Value: quote + v + quote, Kind: token.STRING})
+			pos = next
+		case ast.Expr:
+			compileExpr(ctx, v)
+			t := cb.Get(-1).Type
+			if t.Underlying() != types.Typ[types.String] {
+				cb.Member("string", gox.MemberFlagAutoProperty)
+			}
+			pos = v.End()
+		default:
+			panic("compileStringLitEx TODO: unexpected part")
+		}
+		if notFirst {
+			cb.BinaryOp(gotoken.ADD)
+		} else {
+			notFirst = true
+		}
 	}
 }
 
@@ -903,7 +974,7 @@ func compileCompositeLit(ctx *blockCtx, v *ast.CompositeLit, expected types.Type
 	if t, ok := underlying.(*types.Struct); ok && kind == compositeLitKeyVal {
 		compileStructLitInKeyVal(ctx, v.Elts, t, typ, v)
 		if rec := ctx.recorder(); rec != nil {
-			rec.Type(v, typesutil.NewTypeAndValueForValue(typ, nil))
+			rec.recordCompositeLit(ctx, v, typ)
 		}
 		if hasPtr {
 			ctx.cb.UnaryOp(gotoken.AND)
@@ -920,8 +991,7 @@ func compileCompositeLit(ctx *blockCtx, v *ast.CompositeLit, expected types.Type
 		return
 	}
 	if rec := ctx.recorder(); rec != nil {
-		rec.Type(v.Type, typesutil.NewTypeAndValueForType(typ))
-		rec.Type(v, typesutil.NewTypeAndValueForValue(typ, nil))
+		rec.recordCompositeLit(ctx, v, typ)
 	}
 	switch underlying.(type) {
 	case *types.Slice:
@@ -1017,16 +1087,20 @@ func compileComprehensionExpr(ctx *blockCtx, v *ast.ComprehensionExpr, twoValue
 	end := 0
 	for i := len(v.Fors) - 1; i >= 0; i-- {
 		names := make([]string, 0, 2)
+		defineNames := make([]*ast.Ident, 0, 2)
 		forStmt := v.Fors[i]
 		if forStmt.Key != nil {
 			names = append(names, forStmt.Key.Name)
+			defineNames = append(defineNames, forStmt.Key)
 		} else {
 			names = append(names, "_")
 		}
 		names = append(names, forStmt.Value.Name)
+		defineNames = append(defineNames, forStmt.Value)
 		cb.ForRange(names...)
 		compileExpr(ctx, forStmt.X)
 		cb.RangeAssignThen(forStmt.TokPos)
+		defNames(ctx, defineNames, cb.Scope())
 		if forStmt.Cond != nil {
 			cb.If()
 			if forStmt.Init != nil {
@@ -1108,7 +1182,7 @@ func compileErrWrapExpr(ctx *blockCtx, v *ast.ErrWrapExpr, inFlags int) {
 			retName = "_gop_ret" + strconv.Itoa(i+1)
 		}
 	}
-	sig := types.NewSignature(nil, nil, types.NewTuple(ret...), false)
+	sig := types.NewSignatureType(nil, nil, nil, nil, types.NewTuple(ret...), false)
 	if useClosure {
 		cb.NewClosureWith(sig).BodyStart(pkg)
 	} else {
@@ -1143,7 +1217,7 @@ func compileErrWrapExpr(ctx *blockCtx, v *ast.ErrWrapExpr, inFlags int) {
 			Val(pkg.Import(errorPkgPath).Ref("NewFrame")).
 			Val(err).
 			Val(sprintAst(pkg.Fset, v.X)).
-			Val(pos.Filename).
+			Val(relFile(ctx.relBaseDir, pos.Filename)).
 			Val(pos.Line).
 			Val(currentFuncName).
 			Call(newFrameArgs).
