@@ -32,34 +32,26 @@ import (
 	"github.com/goplus/gox"
 )
 
-func relFile(dir string, file string) string {
-	if rel, err := filepath.Rel(dir, file); err == nil {
-		if rel[0] == '.' {
-			return rel
-		}
-		return "./" + rel
-	}
-	return file
-}
-
-func offsetFileLine(ctx *blockCtx, file string) string {
-	if !filepath.IsAbs(file) {
-		file = filepath.Join(ctx.fileLineWorkingDir, file)
-	}
-	if ret, err := filepath.Rel(ctx.fileLineRoot, file); err == nil {
+func fileLineFile(relBaseDir, absFile string) string {
+	if ret, err := filepath.Rel(relBaseDir, absFile); err == nil {
 		return filepath.ToSlash(ret)
 	}
-	return file
+	return absFile
+}
+
+func relFile(dir string, absFile string) string {
+	if dir != "" {
+		return fileLineFile(dir, absFile)
+	}
+	return absFile
 }
 
 func commentStmt(ctx *blockCtx, stmt ast.Stmt) {
 	if ctx.fileLine {
 		start := stmt.Pos()
 		pos := ctx.fset.Position(start)
-		if ctx.fileLineRoot != "" {
-			pos.Filename = offsetFileLine(ctx, pos.Filename)
-		} else if ctx.relativePath {
-			pos.Filename = relFile(ctx.targetDir, pos.Filename)
+		if ctx.relBaseDir != "" {
+			pos.Filename = fileLineFile(ctx.relBaseDir, pos.Filename)
 		}
 		line := fmt.Sprintf("\n//line %s:%d:1", pos.Filename, pos.Line)
 		comments := &goast.CommentGroup{
@@ -73,10 +65,8 @@ func commentFunc(ctx *blockCtx, fn *gox.Func, decl *ast.FuncDecl) {
 	start := decl.Name.Pos()
 	if ctx.fileLine && start != token.NoPos {
 		pos := ctx.fset.Position(start)
-		if ctx.fileLineRoot != "" {
-			pos.Filename = offsetFileLine(ctx, pos.Filename)
-		} else if ctx.relativePath {
-			pos.Filename = relFile(ctx.targetDir, pos.Filename)
+		if ctx.relBaseDir != "" {
+			pos.Filename = fileLineFile(ctx.relBaseDir, pos.Filename)
 		}
 		var line string
 		if decl.Shadow {
@@ -126,7 +116,7 @@ func compileStmt(ctx *blockCtx, stmt ast.Stmt) {
 		}
 		compileExpr(ctx, v.X, inFlags)
 		if inFlags != 0 && gox.IsFunc(ctx.cb.InternalStack().Get(-1).Type) {
-			ctx.cb.Call(0)
+			ctx.cb.CallWith(0, 0, v.X)
 		}
 	case *ast.AssignStmt:
 		compileAssignStmt(ctx, v)
@@ -327,15 +317,20 @@ func compileRangeStmt(ctx *blockCtx, v *ast.RangeStmt) {
 	}
 	cb := ctx.cb
 	comments, once := cb.BackupComments()
+	defineNames := make([]*ast.Ident, 0, 2)
 	if v.Tok == token.DEFINE {
 		names := make([]string, 1, 2)
 		if v.Key == nil {
 			names[0] = "_"
 		} else {
-			names[0] = v.Key.(*ast.Ident).Name
+			key := v.Key.(*ast.Ident)
+			names[0] = key.Name
+			defineNames = append(defineNames, key)
 		}
 		if v.Value != nil {
-			names = append(names, v.Value.(*ast.Ident).Name)
+			value := v.Value.(*ast.Ident)
+			names = append(names, value.Name)
+			defineNames = append(defineNames, value)
 		}
 		cb.ForRangeEx(names, v)
 		compileExpr(ctx, v.X)
@@ -362,6 +357,9 @@ func compileRangeStmt(ctx *blockCtx, v *ast.RangeStmt) {
 		pos = v.For
 	}
 	cb.RangeAssignThen(pos) // TODO: need NewScope for body
+	if len(defineNames) > 0 {
+		defNames(ctx, defineNames, cb.Scope())
+	}
 	compileStmts(ctx, v.Body.List)
 	cb.SetComments(comments, once)
 	setBodyHandler(ctx)
@@ -376,17 +374,23 @@ func compileForPhraseStmt(ctx *blockCtx, v *ast.ForPhraseStmt) {
 	cb := ctx.cb
 	comments, once := cb.BackupComments()
 	names := make([]string, 1, 2)
+	defineNames := make([]*ast.Ident, 0, 2)
 	if v.Key == nil {
 		names[0] = "_"
 	} else {
 		names[0] = v.Key.Name
+		defineNames = append(defineNames, v.Key)
 	}
 	if v.Value != nil {
 		names = append(names, v.Value.Name)
+		defineNames = append(defineNames, v.Value)
 	}
 	cb.ForRange(names...)
 	compileExpr(ctx, v.X)
 	cb.RangeAssignThen(v.TokPos)
+	if len(defineNames) > 0 {
+		defNames(ctx, defineNames, cb.Scope())
+	}
 	if v.Cond != nil {
 		cb.If()
 		compileExpr(ctx, v.Cond)
@@ -824,7 +828,7 @@ func compileDeclStmt(ctx *blockCtx, expr *ast.DeclStmt) {
 		case token.VAR:
 			for _, spec := range d.Specs {
 				v := spec.(*ast.ValueSpec)
-				loadVars(ctx, v, false)
+				loadVars(ctx, v, d.Doc, false)
 			}
 		default:
 			log.Panicln("TODO: compileDeclStmt - unknown")
