@@ -121,13 +121,14 @@ type Config struct {
 	Filter   func(fs.FileInfo) bool
 	Importer types.Importer
 
-	DontUpdateGoMod     bool
-	DontCheckModChanged bool
-	IgnoreNotatedError  bool
+	// Context represents all things between packages (optional).
+	Context *gox.Context
+
+	IgnoreNotatedError bool
 }
 
-func LoadMod(dir string, gop *env.Gop, conf *Config) (mod *gopmod.Module, err error) {
-	mod, err = gopmod.Load(dir, 0)
+func LoadMod(dir string) (mod *gopmod.Module, err error) {
+	mod, err = gopmod.Load(dir)
 	if err != nil && !NotFound(err) {
 		err = errors.NewWith(err, `gopmod.Load(dir, 0)`, -2, "gopmod.Load", dir, 0)
 		return
@@ -136,22 +137,25 @@ func LoadMod(dir string, gop *env.Gop, conf *Config) (mod *gopmod.Module, err er
 		err = mod.ImportClasses()
 		if err != nil {
 			err = errors.NewWith(err, `mod.RegisterClasses()`, -2, "(*gopmod.Module).RegisterClasses", mod)
-			return
-		}
-		if !conf.DontUpdateGoMod {
-			err = mod.UpdateGoMod(gop, !conf.DontCheckModChanged)
-			if err != nil {
-				err = errors.NewWith(err, `mod.UpdateGoMod(gop, !conf.DontCheckModChanged)`, -2, "(*gopmod.Module).UpdateGoMod", mod, gop, !conf.DontCheckModChanged)
-			}
 		}
 		return
 	}
-	return new(gopmod.Module), nil
+	return gopmod.Default, nil
+}
+
+func hasModfile(mod *gopmod.Module) bool {
+	f := mod.File
+	return f != nil && f.Syntax != nil
 }
 
 // -----------------------------------------------------------------------------
 
 func LoadDir(dir string, conf *Config, genTestPkg bool, promptGenGo ...bool) (out, test *gox.Package, err error) {
+	mod, err := LoadMod(dir)
+	if err != nil {
+		return
+	}
+
 	if conf == nil {
 		conf = new(Config)
 	}
@@ -159,19 +163,10 @@ func LoadDir(dir string, conf *Config, genTestPkg bool, promptGenGo ...bool) (ou
 	if fset == nil {
 		fset = token.NewFileSet()
 	}
-	gop := conf.Gop
-	if gop == nil {
-		gop = gopenv.Get()
-	}
-	mod, err := LoadMod(dir, gop, conf)
-	if err != nil {
-		return
-	}
-
 	pkgs, err := parser.ParseDirEx(fset, dir, parser.Config{
 		ClassKind: mod.ClassKind,
 		Filter:    conf.Filter,
-		Mode:      parser.ParseComments,
+		Mode:      parser.ParseComments | parser.SaveAbsFile,
 	})
 	if err != nil {
 		return
@@ -182,19 +177,22 @@ func LoadDir(dir string, conf *Config, genTestPkg bool, promptGenGo ...bool) (ou
 
 	imp := conf.Importer
 	if imp == nil {
+		gop := conf.Gop
+		if gop == nil {
+			gop = gopenv.Get()
+		}
 		imp = NewImporter(mod, gop, fset)
 	}
 
 	var pkgTest *ast.Package
 	var clConf = &cl.Config{
-		WorkingDir:  dir,
-		Fset:        fset,
-		Importer:    imp,
-		LookupClass: mod.LookupClass,
-		LookupPub:   c2go.LookupPub(mod),
-	}
-	if mod.IsValid() {
-		clConf.FileLineRoot = mod.Root()
+		Fset:           fset,
+		Context:        conf.Context,
+		RelativeBase:   relativeBaseOf(mod),
+		Importer:       imp,
+		IsPkgtStandard: mod.IsPkgtStandard,
+		LookupClass:    mod.LookupClass,
+		LookupPub:      c2go.LookupPub(mod),
 	}
 
 	for name, pkg := range pkgs {
@@ -231,9 +229,23 @@ func LoadDir(dir string, conf *Config, genTestPkg bool, promptGenGo ...bool) (ou
 	return
 }
 
+func relativeBaseOf(mod *gopmod.Module) string {
+	if hasModfile(mod) {
+		return mod.Root()
+	}
+	dir, _ := os.Getwd()
+	return dir
+}
+
 // -----------------------------------------------------------------------------
 
-func LoadFiles(files []string, conf *Config) (out *gox.Package, err error) {
+func LoadFiles(dir string, files []string, conf *Config) (out *gox.Package, err error) {
+	mod, err := LoadMod(dir)
+	if err != nil {
+		err = errors.NewWith(err, `LoadMod(dir)`, -2, "gop.LoadMod", dir)
+		return
+	}
+
 	if conf == nil {
 		conf = new(Config)
 	}
@@ -241,17 +253,7 @@ func LoadFiles(files []string, conf *Config) (out *gox.Package, err error) {
 	if fset == nil {
 		fset = token.NewFileSet()
 	}
-	gop := conf.Gop
-	if gop == nil {
-		gop = gopenv.Get()
-	}
-	mod, err := LoadMod("", gop, conf)
-	if err != nil {
-		err = errors.NewWith(err, `loadMod("", gop, conf)`, -2, "gop.loadMod", "", gop, conf)
-		return
-	}
-
-	pkgs, err := parser.ParseFiles(fset, files, parser.ParseComments)
+	pkgs, err := parser.ParseFiles(fset, files, parser.ParseComments|parser.SaveAbsFile)
 	if err != nil {
 		err = errors.NewWith(err, `parser.ParseFiles(fset, files, parser.ParseComments)`, -2, "parser.ParseFiles", fset, files, parser.ParseComments)
 		return
@@ -260,16 +262,23 @@ func LoadFiles(files []string, conf *Config) (out *gox.Package, err error) {
 		err = errors.NewWith(ErrMultiPackges, `len(pkgs) != 1`, -1, "!=", len(pkgs), 1)
 		return
 	}
+	gop := conf.Gop
+	if gop == nil {
+		gop = gopenv.Get()
+	}
 	for _, pkg := range pkgs {
 		imp := conf.Importer
 		if imp == nil {
 			imp = NewImporter(mod, gop, fset)
 		}
 		clConf := &cl.Config{
-			Fset:        fset,
-			Importer:    imp,
-			LookupClass: mod.LookupClass,
-			LookupPub:   c2go.LookupPub(mod),
+			Fset:           fset,
+			Context:        conf.Context,
+			RelativeBase:   relativeBaseOf(mod),
+			Importer:       imp,
+			LookupClass:    mod.LookupClass,
+			IsPkgtStandard: mod.IsPkgtStandard,
+			LookupPub:      c2go.LookupPub(mod),
 		}
 		out, err = cl.NewPackage("", pkg, clConf)
 		if err != nil {
